''' 
Platform independent setup script for cfxdx docker compose.

Copyright (c) 2021 CloudFabrix Software Inc. All rights reserved.
'''
import os
import sys
import time
import platform
import shutil
from shutil import which


def is_tool(name):
    """Check whether `name` is on PATH and marked as executable."""

    # from whichcraft import which
    
    return which(name) is not None

def INFO(arg):
    print ("[INFO]", arg)

def ERROR(arg):
    print ("[ERROR]", arg)

def FATAL(arg):
    print ("[FATAL]", arg)
    sys.exit(1)

TOP_DIR="~/cfx/"

TOP_DIR=os.path.expanduser(TOP_DIR)

ostype = platform.system()
if ostype:
    ostype = ostype.lower()

check_files = [ 'LICENSE.txt', 'conf.yml', 'docker-compose.yml' ]

for f in check_files:
    if not os.path.isfile(f):
        FATAL("Expecting file {} in current directory, but not found".format(f))

with open('LICENSE.txt', 'r') as fd:
    print (fd.read())
    
time.sleep(1)

if not is_tool("docker"):
    FATAL ("docker not found. Make sure to install docker on this system.")

if not is_tool("docker-compose"):
    FATAL ("docker-compose not found. Make sure to install docker-compose on this system.")

if ostype != "windows":
    if os.system("docker login -u=readonly -p=readonly cfxregistry.cloudfabrix.io 2>/dev/null") != 0:
        FATAL ("Failed to connect to registry. Check network access.")
else:
    if os.system("docker login -u=readonly -p=readonly cfxregistry.cloudfabrix.io 2> nul") != 0:
        FATAL ("Failed to connect to registry. Check network access.")


CFXDX_DIR = "{}/cfxdx/".format(TOP_DIR)
MINIO_DIR = "{}/minio/".format(TOP_DIR)
DIRS = [
        "{}/home/".format(CFXDX_DIR),
        "{}/home/notebooks/".format(CFXDX_DIR),
        "{}/config/".format(CFXDX_DIR),
        "{}/output/".format(CFXDX_DIR),
        "{}/data/cfxdx-data/".format(MINIO_DIR)
    ]

for d in DIRS:
    if not os.path.isdir(d):
        try:
            os.makedirs(d)
        except Exception as e:
            FATAL ("Failed to create directory {}: {}".format(d, e))
  
def conditional_copy(from_file, to_path):
    if not os.path.isfile(to_path):
        shutil.copy(from_file, to_path)

conditional_copy("conf.yml", "{}/config/conf.yml".format(CFXDX_DIR))
shutil.copy("conf_template.yml", "{}/config/conf_template.yml".format(CFXDX_DIR))
conditional_copy("cfx_license_key.yml", "{}/home/cfx_license_key.yml".format(CFXDX_DIR))
shutil.copy("HowTo-cfxdx.ipynb", "{}/home/notebooks/HowTo-cfxdx.ipynb".format(CFXDX_DIR))

INFO ("Downloading docker images")
if os.system("docker-compose pull") != 0:
    FATAL ("Failed to download docker images")

INFO ("Starting Services")
if os.system("docker-compose up --d") != 0:
    FATAL ("Failed to start services")

print()
print ("----------------------------------------------------------------------------")
print ("Connect using browser to http://IPADDRESS:9998/")
print ("----------------------------------------------------------------------------")
print ()
